﻿using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using HslCommunication.Enthernet;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using SlnMesnac.Business.@base;
using SlnMesnac.Config;
using SlnMesnac.Extensions;
using SlnMesnac.Plc;
using SlnMesnac.Repository.service;
using SlnMesnac.TouchSocket;
using SlnMesnac.WPF.Page.Generate;
using SlnMesnac.WPF.Page.IndexPage;
using System;
using System.Windows;

namespace SlnMesnac.WPF.ViewModel
{
    public class MainWindowViewModel: ViewModelBase
    {
        private readonly ILogger<MainWindowViewModel> _logger;
        private readonly ILogger<BaseTaskInfoBusiness> _BaseTaskInfoBusinesslogger;
        private IAirportTaskService _Taskservice;
        private IAGVStateService _AGVStateService;
        private TcpServer _tcpServer;
        //代码生成
        private readonly GenerateControl generateControl = new GenerateControl();
        private MaterialBind indexContent = new MaterialBind();
        //private ChangeType changeType = new ChangeType();
        private AppConfig _appConfig;
        private AirPorthttpClient _airPorthttpClient;

        #region 参数定义
        /// <summary>
        /// PLC设备状态
        /// </summary>
        private int _PlcStatus = 0;
        public int PlcStatus
        {
            get { return _PlcStatus; }
            set { _PlcStatus = value; RaisePropertyChanged(nameof(PlcStatus)); }
        }
        /// <summary>
        /// 箱壳扫码器状态
        /// </summary>
        private int _ShellScannerStatus = 0;
        public int ShellScannerStatus
        {
            get { return _ShellScannerStatus; }
            set { _ShellScannerStatus = value; RaisePropertyChanged(nameof(ShellScannerStatus)); }
        }

        /// <summary>
        /// 内胆扫码器状态
        /// </summary>
        private int _BoldScannerStatus = 0;
        public int BoldScannerStatus
        {
            get { return _BoldScannerStatus; }
            set { _BoldScannerStatus = value; RaisePropertyChanged(nameof(BoldScannerStatus)); }
        }

        public System.Windows.Controls.UserControl _content;

        public System.Windows.Controls.UserControl UserContent
        {
            get { return _content; }

            set
            {
                _content = value;
                RaisePropertyChanged(nameof(UserContent));
            }
        }
        #endregion

        #region 事件定义
        /// <summary>
        /// 界面跳转按钮事件
        /// </summary>
        public RelayCommand<object> ControlOnClickCommand { get; set; }

        /// <summary>
        /// 打开系统键盘
        /// </summary>
        public RelayCommand OpenSystemKeyboardCommand { get; set; }

        /// <summary>
        /// 窗体控制
        /// </summary>
        public RelayCommand<object> FormControlCommand { get; set; }
        #endregion
        /// <summary>
        /// PLC状态刷新
        /// </summary>
        //public void RefreshStatus(object sender, System.Timers.ElapsedEventArgs e)
        //{
        //    PlcAbsractFactory plc = plcPool.GetPlcByKey("plc");
        //    if (plc != null && plc.IsConnected)
        //    {
        //        PlcStatus = 1;
        //    }
        //    else
        //    {
        //        PlcStatus = 2;
        //    }
        //}
        public MainWindowViewModel()
        { 
            _appConfig = App.ServiceProvider.GetService<AppConfig>();
            _logger = App.ServiceProvider.GetService<ILogger<MainWindowViewModel>>();
            _Taskservice = App.ServiceProvider.GetService<IAirportTaskService>();
            _AGVStateService = App.ServiceProvider.GetService<IAGVStateService>();
            _BaseTaskInfoBusinesslogger = App.ServiceProvider.GetService<ILogger<BaseTaskInfoBusiness>>();
            _tcpServer = App.ServiceProvider.GetService<TcpServer>();
            _airPorthttpClient = App.ServiceProvider.GetService<AirPorthttpClient>();
            BaseTaskInfoBusiness.GetInstance(_BaseTaskInfoBusinesslogger, _Taskservice, _AGVStateService, _tcpServer, _airPorthttpClient);
            ControlOnClickCommand = new RelayCommand<object>(obj => ControlOnClick(obj));
            FormControlCommand = new RelayCommand<object>(x => FormControl(x));
            _tcpServer.RefreshStateAction+= (_clientIP, _state) =>
            {
                //if (_clientIP == _appConfig.AMRIP)
                //{
                //    ShellScannerStatus = _state ? 1 : 2;
                //}
            };
            UserContent = indexContent;
        }

        /// <summary>
        ///  窗体控制
        /// </summary>
        /// <param name="obj"></param>
        private void FormControl(object obj)
        {
            try
            {
                string controlType = obj as string;
                switch (controlType)
                {
                    // 关闭当前窗口
                    case "Exit":
                        //Environment.Exit(0);
                        Application.Current.Shutdown();
                        break;
                    case "Generate":
                        UserContent = generateControl;
                        break;
                    //case "ChangeType":
                    //    UserContent = changeType;
                    //    break;
                    // 还原 或者 最大化当前窗口
                    case "Normal":
                        if (Application.Current.MainWindow.WindowState == WindowState.Normal)
                        {
                            Application.Current.MainWindow.WindowState = WindowState.Maximized;
                            break;
                        }
                        if (Application.Current.MainWindow.WindowState == WindowState.Maximized)
                        {
                            Application.Current.MainWindow.WindowState = WindowState.Normal;
                            break;
                        }
                        break;

                    // 最小化当前窗口
                    case "Minimized":
                        Application.Current.MainWindow.WindowState = WindowState.Minimized;
                        break;
                    default:
                        break;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError("窗体控制逻辑异常", ex);
            }
        }

        /// <summary>
        /// 界面跳转
        /// </summary>
        private void ControlOnClick(object obj)
        {
            try
            {
                string info = obj as string;
                switch (info)
                {
                    case "Index":
                        UserContent = indexContent;
                        break;
                    case "ChangeType":
                        UserContent = new ChangeType();
                        break;
                    default: break;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError("界面跳转逻辑异常", ex);
            }
        }

    }
}
